/*
 * CKEditor - The text editor for Internet - http://ckeditor.com
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 */

(function()
{
	var guardDomWalkerNonEmptyTextNode = function( evt )
	{
		if ( evt.to && evt.to.type == CKEDITOR.NODE_TEXT && evt.to.$.length > 0 )
			this.stop();
	};

	var domWalkerEventBridge = function()
	{
		var me = this,
			handler = function( evt ){ me.fire( evt.data.type, evt.data ); me._.actionEvents.push( evt.data ); };
		this._.walker.on( 'up', handler );
		this._.walker.on( 'down', handler );
		this._.walker.on( 'sibling', handler );
	};

	var fireCharacterEvent = function()
	{
		var obj = { type : 'character', character : this._.textNode ? this._.textNode.getText().charAt( this._.offset ) : null };
		this.fire( 'character', obj );
		this._.actionEvents.push( obj );
		return { character : obj.character, events : this._.actionEvents };
	};

	var findDialog = function( editor, startupPage )
	{
		var characterWalker = function( cursorOrTextNode, offset )
		{
			if ( arguments.length < 2 )
				return;

			var isCursor = ( cursorOrTextNode instanceof characterWalker );
			CKEDITOR.tools.extend( this._ || ( this._ = {} ),
				{
					textNode : isCursor ? cursorOrTextNode._.textNode : cursorOrTextNode,
					offset : isCursor ? cursorOrTextNode._.offset : offset,
					actionEvents : null
				} );
			this._.walker = new CKEDITOR.dom.domWalker( this._.textNode );
			CKEDITOR.event.implementOn( this );
			domWalkerEventBridge.call( this );
		};

		characterWalker.prototype = {
			next : function()
			{
				// Clear any previous action events.
				this._.actionEvents = [];

				// If there are more characters in the text node, get it and raise an event.
				if ( this._.offset < this._.textNode.getLength() - 1 )
				{
					this._.offset++;
					return fireCharacterEvent.call( this );
				}

				// If we are at the end of the text node, use dom walker to get the next text node.
				var data = this._.walker.forward( guardDomWalkerNonEmptyTextNode );
				this._.textNode = data.node;
				this._.offset = 0;
				return fireCharacterEvent.call( this );
			},

			back : function()
			{
				// Clear any previous action events.
				this._.actionEvents = [];

				// More characters -> decrement offset and return.
				if ( this._.offset > 0 )
				{
					this._.offset--;
					return fireCharacterEvent.call( this );
				}

				// Start of text node -> use dom walker to get the previous text node.
				var data = this._.walker.reverse( guardDomWalkerNonEmptyTextNode );
				this._.textNode = data.node;
				this._.offset = data.node.length - 1;
				return fireCharacterEvent.call( this );
			}
		};

		var characterRange = function()
		{
		};

		characterRange.prototype = {
		};

		return {
			title : editor.lang.findAndReplace.title,
			resizable : CKEDITOR.DIALOG_RESIZE_NONE,
			minWidth : 400,
			minHeight : 245,
			buttons : [ CKEDITOR.dialog.cancelButton ],		//Cancel button only.
			contents : [
				{
					id : 'find',
					label : editor.lang.findAndReplace.find,
					title : editor.lang.findAndReplace.find,
					accessKey : '',
					elements : [
						{
							type : 'hbox',
							widths : [ '230px', '90px' ],
							children :
							[
								{
									type : 'text',
									id : 'txtFindFind',
									label : editor.lang.findAndReplace.findWhat,
									isChanged : false,
									labelLayout : 'horizontal',
									accessKey : 'F',
								},
								{
									type : 'button',
									align : 'left',
									style : 'width:100%',
									label : editor.lang.findAndReplace.find,
									onClick : function()
									{
										alert( editor.lang.findAndReplace.notFoundMsg );
									}
								},
							]
						},
						{
							type : 'checkbox',
							id : 'txtFindCaseChk',
							isChanged : false,
							style : 'margin-top:28px',
							label : editor.lang.findAndReplace.matchCase,
						},
						{
							type : 'checkbox',
							id : 'txtFindWordChk',
							isChanged : false,
							label : editor.lang.findAndReplace.matchWord,
						},
						{
							type : 'checkbox',
							id : 'txtFindCyclic',
							isChanged : false,
							checked : true,
							label : editor.lang.findAndReplace.matchCyclic,
						},
					]
				},
				{
					id : 'replace',
					label : editor.lang.findAndReplace.replace,
					accessKey : 'M',
					elements : [
						{
							type : 'hbox',
							widths : [ '230px', '90px' ],
							children :
							[
								{
									type : 'text',
									id : 'txtFindReplace',
									label : editor.lang.findAndReplace.findWhat,
									isChanged : false,
									labelLayout : 'horizontal',
									accessKey : 'F',
								},
								{
									type : 'button',
									align : 'left',
									style : 'width:100%',
									label : editor.lang.findAndReplace.replace,
									onClick : function()
									{
										alert( editor.lang.findAndReplace.notFoundMsg );
									}
								},
							]
						},
						{
							type : 'hbox',
							widths : [ '230px', '90px' ],
							children :
							[
								{
									type : 'text',
									id : 'txtReplace',
									label : editor.lang.findAndReplace.replaceWith,
									isChanged : false,
									labelLayout : 'horizontal',
									accessKey : 'R',
								},
								{
									type : 'button',
									align : 'left',
									style : 'width:100%',
									label : editor.lang.findAndReplace.replaceAll,
									isChanged : false,
									onClick : function()
									{
										alert( editor.lang.findAndReplace.notFoundMsg );
									}
								},
							]
						},
						{
							type : 'checkbox',
							id : 'txtReplaceCaseChk',
							isChanged : false,
							label : editor.lang.findAndReplace.matchCase,
						},
						{
							type : 'checkbox',
							id : 'txtReplaceWordChk',
							isChanged : false,
							label : editor.lang.findAndReplace.matchWord,
						},
						{
							type : 'checkbox',
							id : 'txtReplaceCyclic',
							isChanged : false,
							checked : true,
							label : editor.lang.findAndReplace.matchCyclic,
						},
					]
				}
			],
			onShow : function()
			{
				if ( startupPage == 'replace' )
					this.getContentElement( 'replace', 'txtFindReplace' ).focus();
				else
					this.getContentElement( 'find', 'txtFindFind' ).focus();
			}
		};
	};

	CKEDITOR.dialog.add( 'find', function( editor ){
			return findDialog( editor, 'find' )
		}
	);

	CKEDITOR.dialog.add( 'replace', function( editor ){
			return findDialog( editor, 'replace' )
		}
	);
})();
